@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -787,70 +790,86 @@ IOFBRebuild( IOFBConnectRef connectRef )
 }
 
 static void
-IOFBInterestCallback( void * refcon, io_service_t service,
-                      natural_t messageType, void * messageArgument )
+IOFBProcessConnectChange( IOFBConnectRef connectRef )
 {
-    IOFBConnectRef		connectRef = (IOFBConnectRef) refcon;
     IOReturn			err;
     IODisplayModeID		mode = 0;
     IOIndex			depth;
     IODisplayModeInformation	info;
     UInt32			value;
     UInt32			previousState, previous4By3, previousProduct, previousVendor;
+    
+    IOFBGetAttributeForFramebuffer( connectRef->connect, MACH_PORT_NULL,
+				    kConnectionChanged, &value );
+
+    previousState   = connectRef->state;
+    previous4By3    = connectRef->make4By3;
+    previousProduct = connectRef->displayProduct;
+    previousVendor  = connectRef->displayVendor;
+
+    IOFBUpdateConnectState( connectRef );
+
+    IOFBRebuild( connectRef );
+
+    if( kIOFBConnectStateOnline & previousState & connectRef->state) do {
+
+	if( previousProduct != connectRef->displayProduct)
+	    break;
+	if( previousVendor != connectRef->displayVendor)
+	    break;
+	if( previous4By3 != connectRef->make4By3)
+	    break;
+
+	err = IOFBGetCurrentDisplayModeAndDepth( connectRef->connect, &mode, &depth );
+	if( kIOReturnSuccess == err)
+	    err = IOFBGetDisplayModeInformation( connectRef->connect, mode, &info );
+	if( (kIOReturnSuccess != err) || (0 == (info.flags & kDisplayModeValidFlag))) {
+	    mode = 0;
+	    break;
+	}
+
+    } while( false );
+
+    if( !mode) {
+	mode  = connectRef->defaultMode;
+	depth = connectRef->defaultDepth;
+    }
+
+    if( connectRef->make4By3 && connectRef->default4By3Mode) {
+	err = IOFBGetDisplayModeInformation( connectRef->connect, mode, &info );
+	if( (kIOReturnSuccess == err) 
+	    && ratioOver(((float)info.nominalWidth) / ((float)info.nominalHeight), 4.0 / 3.0) > 1.03125) {
+	    mode = connectRef->default4By3Mode;
+	}
+    }
+
+    err = IOFBSetDisplayModeAndDepth( connectRef->connect, mode, depth );
+
+    connectRef->clientCallbacks->ConnectionChange(connectRef->clientCallbackRef, (void *) NULL);
+
+}
+
+static void
+IOFBInterestCallback( void * refcon, io_service_t service,
+                      natural_t messageType, void * messageArgument )
+{
+    IOFBConnectRef connectRef = (IOFBConnectRef) refcon;
+    IOFBConnectRef next;
 
     switch( messageType) {
 
       case kIOMessageServiceIsSuspended:
 
-    
-        IOFBGetAttributeForFramebuffer( connectRef->connect, MACH_PORT_NULL,
-                                        kConnectionChanged, &value );
-    
-        previousState   = connectRef->state;
-        previous4By3    = connectRef->make4By3;
-        previousProduct = connectRef->displayProduct;
-        previousVendor  = connectRef->displayVendor;
-    
-        IOFBUpdateConnectState( connectRef );
+	next = connectRef;
+	do {
 
-        IOFBRebuild( connectRef );
+	    IOFBProcessConnectChange( next );
+	    next = next->nextDependent;
 
-        if( kIOFBConnectStateOnline & previousState & connectRef->state) do {
-
-            if( previousProduct != connectRef->displayProduct)
-                break;
-            if( previousVendor != connectRef->displayVendor)
-                break;
-            if( previous4By3 != connectRef->make4By3)
-                break;
-
-            err = IOFBGetCurrentDisplayModeAndDepth( connectRef->connect, &mode, &depth );
-            if( kIOReturnSuccess == err)
-                err = IOFBGetDisplayModeInformation( connectRef->connect, mode, &info );
-            if( (kIOReturnSuccess != err) || (0 == (info.flags & kDisplayModeValidFlag))) {
-                mode = 0;
-                break;
-            }
-    
-        } while( false );
-
-        if( !mode) {
-            mode  = connectRef->defaultMode;
-            depth = connectRef->defaultDepth;
-        }
-    
-        if( connectRef->make4By3 && connectRef->default4By3Mode) {
-            err = IOFBGetDisplayModeInformation( connectRef->connect, mode, &info );
-            if( (kIOReturnSuccess == err) 
-              && ratioOver(((float)info.nominalWidth) / ((float)info.nominalHeight), 4.0 / 3.0) > 1.03125) {
-                mode = connectRef->default4By3Mode;
-            }
-        }
-    
-        err = IOFBSetDisplayModeAndDepth( connectRef->connect, mode, depth );
-
-        connectRef->clientCallbacks->ConnectionChange(connectRef->clientCallbackRef, (void *) NULL);
+	} while( next && (next != connectRef) );
+        break;
 
+      default:
         break;
     }
 }
